#ifndef __ALCD_THREAD_CPP
#define __ALCD_THREAD_CPP

#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <math.h>
#include <time.h>

#include "i18n.h"
#include "alcd-thread.h"
#include <vdr/tools.h>
#include <vdr/config.h>

#include <vdr/plugin.h>
#include <vdr/status.h>
#include <vdr/channels.h>

extern int PowerButton;
extern int WaitTime;
extern int RecordLED;
extern int LEDBrightness;
extern int LEDBrightnessOnExit;

///////////////////////////////////////////////////
// activy300 LCD plugin for VDR
//
// Original written by:
//
// Meinrad Sauter <meinrad.sauter@gmx.de>
// andreas 'randy' weinberger <vdr@smue.org>
//
// rearranged, beautified and rewritten by:
//
// Markus Geisler <geisler@primusnetz.de>
//

/////////////////////////////////////////////////////////
// Konstruktor

aLCD::aLCD()
{
	// new instance of LCD
	myLCD = new activyLCD();

	// waittimer for status messages
	wait = 0;
   	refresh = 0;

  	stop = false;
  	title = false;

        linesize0 = 0;
        leftborder0 = 0;
        rightborder0 = 0;
        currentborder0 = 0;
        linesize1 = 0;
        leftborder1 = 0;
        rightborder1 = 0;
        currentborder1 = 0;
        scrolling1 = 0;

        LCDLine0 = strdup("");
        LCDLine1 = strdup("");
        LCDoldLine0 = strdup("");
        LCDoldLine1 = strdup("");
        LCDrecLine0 = strdup("");
        LCDrecLine1 = strdup("");
        LCDvolLine0 = strdup("");
        LCDvolLine1 = strdup("");
        LCDplayLine0 = strdup("");
        LCDplayLine1 = strdup("");
        LCDmenuLine0 = strdup("");
        LCDmenuLine1 = strdup("");
        LCDChannelLine0 = strdup("");
        LCDChannelLine1 = strdup("");

  	memset(line0,' ',20);
  	memset(line1,' ',20);
  	memset(oldline0,' ',20);
  	memset(oldline1,' ',20);
  	memset(templine0,' ',20);
  	memset(templine1,' ',20);

  	myLCD->LCDInit();  

  	refresh = 1;
}

/////////////////////////////////////////////////////////
// Destruktor

aLCD::~aLCD()
{
  	stop = true;
  	while (running) usleep(100000);
  	myLCD->LCDClear();
  	myLCD->SetPowerButton(1);
  	myLCD->SetLEDBrightness(LEDBrightnessOnExit);
  	myLCD->SerialPortClose();
}

/////////////////////////////////////////////////////////
// Reset Class Values

void aLCD::reset()
{
        linesize0 = 0;
        leftborder0 = 0;
        rightborder0 = 0;
        currentborder0 = 0;
        linesize1 = 0;
        leftborder1 = 0;
        rightborder1 = 0;
        currentborder1 = 0;
        scrolling1 = 0;
}

/////////////////////////////////////////////////////////
// Set Channel Name

void aLCD::SetChannelName (const char *Name)
{
	lcdmodus = 0;
	reset();

	if(LCDChannelLine0)
	{
		free(LCDChannelLine0);
	}
	LCDChannelLine0 = strdup(myLCD->ReplaceUmlaute(Name));
}

/////////////////////////////////////////////////////////
// Set Programm

void aLCD::SetProgramme (const char *PresentTitle)
{
	lcdmodus = 0;
	reset();

	if(LCDChannelLine1)
	{
		free(LCDChannelLine1);
	}
	LCDChannelLine1 = strdup(myLCD->ReplaceUmlaute(PresentTitle));
}

/////////////////////////////////////////////////////////
// Set Recording

void aLCD::SetRecording(int card, const char *Name)
{
	char output[30];
	char *replaced;

  	if (Name != NULL) 
	{
    		switch (RecordLED) 
		{
      			case 1:
			snprintf(output,2, "%s", MESSAGELED_BLINK_ON);
  			myLCD->SendCommandToLCD(output,2 );
			snprintf(output,2, "%s", BASICLED_OFF);
  			myLCD->SendCommandToLCD(output,2 );
        		break;

      			case 2:
			snprintf(output,2, "%s",BASICLED_ON);
  			myLCD->SendCommandToLCD(output,2 );
			snprintf(output,2, "%s",MESSAGELED_OFF);
  			myLCD->SendCommandToLCD(output,2 );
        		break;

    		};

   		snprintf(RecordName,41,"%s", Name);
    		snprintf(templine0,21,tr("Recording started:"));

		if(LCDrecLine0)
		{
			free(LCDrecLine0);
		}
		LCDrecLine0 = strdup(templine0);

    		replaced = myLCD->ReplaceUmlaute(RecordName);
    		snprintf(templine1,21,"%-20s",replaced);

		if(LCDrecLine1)
		{
			free(LCDrecLine1);
		}
		LCDrecLine1 = strdup(templine1);
  	} 
	else 
	{
    		snprintf(output,2, "%s", MESSAGELED_OFF);
  		myLCD->SendCommandToLCD(output,2 );
    		snprintf(output,2, "%s", BASICLED_OFF);
  		myLCD->SendCommandToLCD(output,2 );

    		snprintf(templine0,21,tr("Recording stopped:"));

		if(LCDrecLine0)
		{
			free(LCDrecLine0);
		}
		LCDrecLine0 = strdup(templine0);

    		replaced = myLCD->ReplaceUmlaute(RecordName);
    		snprintf(templine1,21,"%-20s",replaced);

  		if(LCDrecLine1)
		{
			free(LCDrecLine1);
		}
		LCDrecLine1 = strdup(templine1);
	};
  	lcdmodus = 1;
}

/////////////////////////////////////////////////////////
// Replay Control

void aLCD::ReplayControl(cControl *Control, const char *Name)
{
	reset();

	char *replaced;

  	if (Name == NULL) 
	{
    		lcdmodus = 0;
  	} 
	else 
	{
    		lcdmodus = 3;
    		PlayControl = Control;
    		replaced = myLCD->ReplaceUmlaute(Name);

		if(LCDplayLine0)
		{
			free(LCDplayLine0);
		}

		if(LCDplayLine1)
		{
			free(LCDplayLine1);
		}

		LCDplayLine0 = strdup(replaced);
		LCDplayLine1 = strdup("                  > ");
  	}
}

/////////////////////////////////////////////////////////
// Menu Control

void aLCD::SetCurrentItem(const char *Text)
{
	reset();

	char *replaced;

  	if (Text == NULL) 
	{
    		lcdmodus = 0;
  	} 
	else 
	{
		lcdmodus = 4;
	    	replaced = myLCD->ReplaceUmlaute(Text);

		if(LCDmenuLine1)
		{
			free(LCDmenuLine1);
		}

		LCDmenuLine1 = strdup(replaced);
	}
}

/////////////////////////////////////////////////////////
// Status Information

void aLCD::SetMessageText(const char *Text)
{
	reset();

	if(LCDoldLine0)
	{
		free(LCDoldLine0);
	}

	if(LCDmenuLine0)
	{
		LCDoldLine0 = strdup(LCDmenuLine0);
		free(LCDmenuLine0);
		LCDmenuLine0 = strdup(Text);
		sleep(1);
		free(LCDmenuLine0);
		LCDmenuLine0 = strdup(LCDoldLine0);
	}
}

/////////////////////////////////////////////////////////
// Set Title

void aLCD::SetTitle(const char *Text)
{
	reset();

	if(LCDoldLine0)
	{
		free(LCDoldLine0);
	}

	if(LCDmenuLine0)
	{
		LCDoldLine0 = strdup(LCDmenuLine0);
		free(LCDmenuLine0);
		LCDmenuLine0 = strdup(Text);
	}
}


/////////////////////////////////////////////////////////
// Menu Control

void aLCD::SetOsdClear()
{
//	lcdmodus=0;

	reset();
}

/////////////////////////////////////////////////////////
// Set Volume

void aLCD::SetVolume (int volume, bool muted)
{
  	percent = (int) lround(volume/14); // 1 -> 18
  	if(muted)
  	{
    		snprintf(templine1,21,"[    -- MUTE --    ]");

  		if(LCDvolLine1)
		{
			free(LCDvolLine1);
		}

		LCDvolLine1 = strdup(templine1);
	} 
	else 
	{
    		SetVolumeBar(percent);
  	}
  	oldmodus=lcdmodus;
  	lcdmodus=2;
}

/////////////////////////////////////////////////////////
// Set Volume Bar

void aLCD::SetVolumeBar(int percent)
{
  	int temp;
  	char vol_temp[18];
  	for (temp=1;temp<percent+1;temp++) 
	{
    		strcat(vol_temp, "#");
  	}
  	snprintf(templine1,21,"[%-18s]", vol_temp);

	if(LCDvolLine1)
	{
		free(LCDvolLine1);
	}

	LCDvolLine1 = strdup(templine1);
}

/////////////////////////////////////////////////////////
// Set Progress Bar

void aLCD::SetProgressBar(int percent)
{
  	int temp;
  	char prog_temp[15];
    	for (temp=1;temp<percent+1;temp++) 
	{
    		strcat(prog_temp, "#");
  	}
  	snprintf(templine1,21,"[%-15s]", prog_temp);
	LCDChannelLine1 = strdup(templine1);
}

/////////////////////////////////////////////////////////
// Thread Action

void aLCD::Action(void)
{
  	running=true;

  	while(!stop)
	{
    		gettimeofday(&now,NULL);
    		struct tm uhr;
    		time_t val;

    		switch (lcdmodus) 
		{ 
			//
			// case 0 : Channeldisplay
			//

      			case 0: 
			val = time(NULL);
			localtime_r(&val,&uhr);
			snprintf(templine0,14,"%s",LCDChannelLine0);
        		snprintf(line0,21,"%-14s %02d:%02d", templine0, uhr.tm_hour, uhr.tm_min);
        		snprintf(line1,21,"%-20s", LCDChannelLine1);

			linesize1 = strlen(LCDChannelLine1);

			if(linesize1 > 20)
			{
				if(scrolling1 == 0)
				{
					scrolling1 = 1;
				}

				// leftborder = linkester Index fuer Anzeige (Scrolling)
				leftborder1 = 0;
				// rightborder = rechtester Index fuer Anzeige (Scrolling)
				rightborder1 = linesize1 - 21;
				
        			snprintf(line1,21,"%-20s", &LCDChannelLine1[currentborder1]);

				if((currentborder1 <= rightborder1) && (scrolling1 == 1))
				{
					// scrolling = 1 -> rightscrolling
					scrolling1 = 1;
				}
				else
				{
					// scrolling = 2 -> leftscrolling
					scrolling1 = 2;
				}

				if((currentborder1 > leftborder1) && (scrolling1 == 2))
				{
					// scrolling = 2 -> leftscrolling
					scrolling1 = 2;
				}
				else
				{
					// scrolling = 1 -> rightscrolling
					scrolling1 = 1;
				}

				if(scrolling1 == 1)
				{
					currentborder1++;
				}

				if(scrolling1 == 2)
				{
					currentborder1--;
				}
			}
			else
			{
				scrolling1 = 0;
			}

			wait = 0;
        		break;

			//
			// case 1 : display recording
			//

      			case 1: 
        		snprintf(line0,21,"%-20s", LCDrecLine0);
        		snprintf(line1,21,"%-20s", LCDrecLine1);
        		wait++;
        		if(wait>(WaitTime*10)) 
			{
          			snprintf(line1,21,"%-20s", LCDChannelLine1);
          			lcdmodus = 0; // switch back to time / chan display
          			wait = 0;
        		};
			break;

			//
			// case 2 : display volume
			//

      			case 2: 
			val = time(NULL);
			localtime_r(&val,&uhr);
        		snprintf(line0,21,"%-14s %02d:%02d", LCDChannelLine0, uhr.tm_hour, uhr.tm_min);
        		snprintf(line1,21,"%-20s", LCDvolLine1);
        		wait++;
        		if(wait>(WaitTime*10)) 
			{
          			snprintf(line1,21,"%-20s", LCDvolLine1);
          			lcdmodus = 0; // switch back to time / chan display
          			wait = 0;
        		};
			break;

			//
			// case 3 : display replay information
			//

      			case 3: 
        		char PlaySymbol[3];
        		char IndexTime[17];
        		int Current, Total, Speed;
        		bool Play, Forward;
        		if (PlayControl->GetReplayMode(Play,Forward,Speed)) 
			{
          			if (Play && Speed == -1)     snprintf(PlaySymbol,2,"%2s","> "); // Normal Replay
          			if (!Play && Speed == -1)    snprintf(PlaySymbol,2,"%2s","||"); // Pause
          			if (Forward && Speed != -1)  snprintf(PlaySymbol,2,"%2s",">>"); // Fast Forward
          			if (!Forward && Speed != -1) snprintf(PlaySymbol,2,"%2s","<<"); // Fast Backward
        		}
        		if (PlayControl->GetIndex(Current,Total)) 
			{
	  			snprintf(IndexTime,18,"%-17s",IndexToHMSF(Current,false));
          			snprintf(templine1,21,"%-17s %-2s",IndexTime, PlaySymbol);
        		}

        		snprintf(line0,21,"%s", LCDplayLine0);
        		snprintf(line1,21,"%s", templine1);

			linesize0 = strlen(LCDplayLine0);

			if(linesize0 > 20)
			{
				if(scrolling0 == 0)
				{
					scrolling0 = 1;
				}

				// leftborder = linkester Index fuer Anzeige (Scrolling)
				leftborder0 = 0;
				// rightborder = rechtester Index fuer Anzeige (Scrolling)
				rightborder0 = linesize0 - 21;
				
        			snprintf(line0,21,"%s", &LCDplayLine0[currentborder0]);

				if((currentborder0 <= rightborder0) && (scrolling0 == 1))
				{
					// scrolling = 1 -> rightscrolling
					scrolling0 = 1;
				}
				else
				{
					// scrolling = 2 -> leftscrolling
					scrolling0 = 2;
				}

				if((currentborder0 > leftborder0) && (scrolling0 == 2))
				{
					// scrolling = 2 -> leftscrolling
					scrolling0 = 2;
				}
				else
				{
					// scrolling = 1 -> rightscrolling
					scrolling0 = 1;
				}

				if(scrolling0 == 1)
				{
					currentborder0++;
				}

				if(scrolling0 == 2)
				{
					currentborder0--;
				}
			}
			else
			{
				scrolling0 = 0;
			}

			break;

			//
			// case 4 : display menu
			//

      			case 4:
        		snprintf(line0,21,"%s", LCDmenuLine0);
        		snprintf(line1,21,"%s", LCDmenuLine1);

			linesize0 = strlen(LCDmenuLine0);

			if(linesize0 > 20)
			{
				if(scrolling0 == 0)
				{
					scrolling0 = 1;
				}

				// leftborder = linkester Index fuer Anzeige (Scrolling)
				leftborder0 = 0;
				// rightborder = rechtester Index fuer Anzeige (Scrolling)
				rightborder0 = linesize0 - 21;
				
        			snprintf(line0,21,"%s", &LCDmenuLine0[currentborder0]);

				if((currentborder0 <= rightborder0) && (scrolling0 == 1))
				{
					// scrolling = 1 -> rightscrolling
					scrolling0 = 1;
				}
				else
				{
					// scrolling = 2 -> leftscrolling
					scrolling0 = 2;
				}

				if((currentborder0 > leftborder0) && (scrolling0 == 2))
				{
					// scrolling = 2 -> leftscrolling
					scrolling0 = 2;
				}
				else
				{
					// scrolling = 1 -> rightscrolling
					scrolling0 = 1;
				}

				if(scrolling0 == 1)
				{
					currentborder0++;
				}

				if(scrolling0 == 2)
				{
					currentborder0--;
				}
			}
			else
			{
				scrolling0 = 0;
			}

			linesize1 = strlen(LCDmenuLine1);

			if(linesize1 > 20)
			{
				if(scrolling1 == 0)
				{
					scrolling1 = 1;
				}

				// leftborder = linkester Index fuer Anzeige (Scrolling)
				leftborder1 = 0;
				// rightborder = rechtester Index fuer Anzeige (Scrolling)
				rightborder1 = linesize1 - 21;
				
        			snprintf(line1,21,"%-20s", &LCDmenuLine1[currentborder1]);

				if((currentborder1 <= rightborder1) && (scrolling1 == 1))
				{
					// scrolling = 1 -> rightscrolling
					scrolling1 = 1;
				}
				else
				{
					// scrolling = 2 -> leftscrolling
					scrolling1 = 2;
				}

				if((currentborder1 > leftborder1) && (scrolling1 == 2))
				{
					// scrolling = 2 -> leftscrolling
					scrolling1 = 2;
				}
				else
				{
					// scrolling = 1 -> rightscrolling
					scrolling1 = 1;
				}

				if(scrolling1 == 1)
				{
					currentborder1++;
				}

				if(scrolling1 == 2)
				{
					currentborder1--;
				}
			}
			else
			{
				scrolling1 = 0;
			}

			wait = 0;
        		break;
    		}
    	
    		if ((strcmp(line0,oldline0)!=0 && refresh) || (strcmp(line1,oldline1)!=0 && refresh)) 
		{
      			memcpy(oldline0,line0,21);
      			memcpy(oldline1,line1,21); 
      			myLCD->SetLCDLines(line0,line1);
    		}

    		usleep(100000-(now.tv_usec%100000));
  	}
  	running=false;
}

#endif
